@@ -1,21 +1,20 @@
+import json
+import logging
+import os
+import pickle
 import textwrap
+import threading
+import warnings
 
 import google.auth as gauth
 import google.auth.compute_engine
 import google.auth.credentials
 import google.auth.exceptions
+import requests
+from google.auth.transport.requests import Request
+from google.oauth2 import service_account
 from google.oauth2.credentials import Credentials
 from google_auth_oauthlib.flow import InstalledAppFlow
-from google.oauth2 import service_account
-from google.auth.transport.requests import Request
-import json
-import requests
-import os
-import pickle
-import requests
-import threading
-import warnings
-import logging
 
 logger = logging.getLogger("gcsfs.credentials")
 
@@ -140,21 +139,24 @@ class GoogleCredentials:
         Parameters
         ----------
         token: str, dict or Credentials
-            If a str, try to load as a Service file, or next as a JSON; if
+            If a str and a valid file name, try to load as a Service file, or next as a JSON;
+            if not a valid file name, assume it's a valid raw (non-renewable/session) token, and pass to Credentials. If
             dict, try to interpret as credentials; if Credentials, use directly.
         """
         if isinstance(token, str):
-            if not os.path.exists(token):
-                raise FileNotFoundError(token)
-            try:
-                # is this a "service" token?
-                self._connect_service(token)
-                return
-            except:  # noqa: E722
-                # TODO: catch specific exceptions
-                # some other kind of token file
-                # will raise exception if is not json
-                token = json.load(open(token))
+            if os.path.exists(token):
+                try:
+                    # is this a "service" token?
+                    self._connect_service(token)
+                    return
+                except:  # noqa: E722
+                    # TODO: catch specific exceptions
+                    # some other kind of token file
+                    # will raise exception if is not json
+                    with open(token) as data:
+                        token = json.load(data)
+            else:
+                token = Credentials(token)
         if isinstance(token, dict):
             credentials = self._dict_to_credentials(token)
         elif isinstance(token, google.auth.credentials.Credentials):
@@ -178,7 +180,8 @@ class GoogleCredentials:
                     return  # repeat to avoid race (but don't want lock in common case)
                 logger.debug("GCS refresh")
                 self.credentials.refresh(req)
-                self.apply(self.heads)
+                # https://github.com/fsspec/filesystem_spec/issues/565
+                self.credentials.apply(self.heads)
 
     def apply(self, out):
         """Insert credential headers in-place to a dictionary"""
