@@ -1,7 +1,7 @@
-import datetime
 import io
 import os
 from builtins import FileNotFoundError
+from datetime import datetime, timezone
 from itertools import chain
 from unittest import mock
 from urllib.parse import parse_qs, unquote, urlparse
@@ -26,12 +26,27 @@ TEST_PROJECT = gcsfs.tests.settings.TEST_PROJECT
 TEST_REQUESTER_PAYS_BUCKET = gcsfs.tests.settings.TEST_REQUESTER_PAYS_BUCKET
 
 
-def test_simple(gcs):
-    assert not GoogleCredentials.tokens
+def test_simple(gcs, monkeypatch):
+    monkeypatch.setattr(GoogleCredentials, "tokens", None)
     gcs.ls(TEST_BUCKET)  # no error
     gcs.ls("/" + TEST_BUCKET)  # OK to lead with '/'
 
 
+def test_dircache_filled(gcs):
+    assert not dict(gcs.dircache)
+    gcs.ls(TEST_BUCKET)
+    assert len(gcs.dircache) == 1
+    gcs.dircache[TEST_BUCKET][0]["CHECK"] = True
+    out = gcs.ls(TEST_BUCKET, detail=True)
+    assert [o for o in out if o.get("CHECK", None)]
+
+    gcs.invalidate_cache()
+    assert not dict(gcs.dircache)
+
+    gcs.find(TEST_BUCKET)
+    assert len(gcs.dircache)
+
+
 def test_many_connect(docker_gcs):
     from multiprocessing.pool import ThreadPool
 
@@ -125,9 +140,12 @@ def test_info(gcs):
     gcs.touch(a)
     assert gcs.info(a) == gcs.ls(a, detail=True)[0]
 
-    today = datetime.date.today().isoformat()
+    today = datetime.utcnow().date().isoformat()
     assert gcs.created(a).isoformat().startswith(today)
     assert gcs.modified(a).isoformat().startswith(today)
+    # Check conformance with expected info attribute names.
+    assert gcs.info(a)["ctime"] == gcs.created(a)
+    assert gcs.info(a)["mtime"] == gcs.modified(a)
 
 
 def test_ls2(gcs):
@@ -180,9 +198,8 @@ def test_rm(gcs):
     assert gcs.exists(a)
     gcs.rm(a)
     assert not gcs.exists(a)
-    # silently ignored for now
-    # with pytest.raises((OSError, IOError)):
-    #    gcs.rm(TEST_BUCKET + "/nonexistent")
+    with pytest.raises((OSError, IOError)):
+        gcs.rm(TEST_BUCKET + "/nonexistent")
     with pytest.raises((OSError, IOError)):
         gcs.rm("nonexistent")
 
@@ -267,25 +284,33 @@ def test_ls_detail(gcs):
     assert all(isinstance(item, dict) for item in L)
 
 
+@pytest.mark.parametrize("refresh", (False, True))
+def test_ls_refresh(gcs, refresh):
+    with mock.patch.object(gcs, "invalidate_cache") as mock_invalidate_cache:
+        gcs.ls(TEST_BUCKET, refresh=refresh)
+    assert mock_invalidate_cache.called is refresh
+
+
 def test_gcs_glob(gcs):
     fn = TEST_BUCKET + "/nested/file1"
     assert fn not in gcs.glob(TEST_BUCKET + "/")
     assert fn not in gcs.glob(TEST_BUCKET + "/*")
-    assert fn in gcs.glob(TEST_BUCKET + "/nested/")
+    assert fn not in gcs.glob(TEST_BUCKET + "/nested/")
     assert fn in gcs.glob(TEST_BUCKET + "/nested/*")
     assert fn in gcs.glob(TEST_BUCKET + "/nested/file*")
     assert fn in gcs.glob(TEST_BUCKET + "/*/*")
     assert fn in gcs.glob(TEST_BUCKET + "/**")
-    assert fn in gcs.glob(TEST_BUCKET + "/**1")
+    assert fn in gcs.glob(TEST_BUCKET + "/**/*1")
     assert all(
         f in gcs.find(TEST_BUCKET)
         for f in gcs.glob(TEST_BUCKET + "/nested/*")
         if gcs.isfile(f)
     )
+    # the following is no longer true since the glob method list the root path
     # Ensure the glob only fetches prefixed folders
-    gcs.dircache.clear()
-    gcs.glob(TEST_BUCKET + "/nested**1")
-    assert all(d.startswith(TEST_BUCKET + "/nested") for d in gcs.dircache)
+    # gcs.dircache.clear()
+    # gcs.glob(TEST_BUCKET + "/nested**1")
+    # assert all(d.startswith(TEST_BUCKET + "/nested") for d in gcs.dircache)
     # the following is no longer true as of #437
     # gcs.glob(TEST_BUCKET + "/test*")
     # assert TEST_BUCKET + "/test" in gcs.dircache
@@ -396,6 +421,19 @@ def test_move(gcs):
     assert not gcs.exists(fn)
 
 
+@pytest.mark.parametrize("slash_from", ([False, True]))
+def test_move_recursive(gcs, slash_from):
+    # See issue #489
+    dir_from = TEST_BUCKET + "/nested"
+    if slash_from:
+        dir_from += "/"
+    dir_to = TEST_BUCKET + "/new_name"
+
+    gcs.mv(dir_from, dir_to, recursive=True)
+    assert not gcs.exists(dir_from)
+    assert gcs.ls(dir_to) == [dir_to + "/file1", dir_to + "/file2", dir_to + "/nested2"]
+
+
 def test_cat_file(gcs):
     fn = TEST_BUCKET + "/test/accounts.1.json"
     data = gcs.cat_file(fn)
@@ -477,6 +515,19 @@ def test_get_put_file_in_dir(protocol, gcs):
         assert gcs.cat(protocol + TEST_BUCKET + "/temp_dir/accounts.1.json") == data1
 
 
+@pytest.mark.parametrize("protocol", ["", "gs://", "gcs://"])
+def test_get_file_to_current_working_directory(monkeypatch, protocol, gcs):
+    fn = protocol + TEST_BUCKET + "/temp"
+    gcs.pipe(fn, b"hello world")
+
+    with tempdir() as dn:
+        os.makedirs(dn)
+        monkeypatch.chdir(dn)
+        gcs.get_file(fn, "temp")
+        with open("temp", mode="rb") as f:
+            assert f.read() == b"hello world"
+
+
 def test_special_characters_filename(gcs: GCSFileSystem):
     special_filename = """'!"`#$%&'()+,-.<=>?@[]^_{}~/'"""
     full_path = TEST_BUCKET + "/" + special_filename
@@ -819,8 +870,10 @@ def test_bigger_than_block_read(gcs):
 
 
 def test_current(gcs):
+    from gcsfs.tests import conftest
+
     assert GCSFileSystem.current() is gcs
-    gcs2 = GCSFileSystem(endpoint_url=gcs._endpoint, default_location=None)
+    gcs2 = GCSFileSystem(**conftest.params)
     assert gcs2.session is gcs.session
 
 
@@ -837,6 +890,36 @@ def test_array(gcs):
         assert out == b"A" * 1000
 
 
+def test_content_type_set(gcs):
+    fn = TEST_BUCKET + "/content_type"
+    with gcs.open(fn, "wb", content_type="text/html") as f:
+        f.write(b"<html>")
+    assert gcs.info(fn)["contentType"] == "text/html"
+
+
+def test_content_type_guess(gcs):
+    fn = TEST_BUCKET + "/content_type.txt"
+    with gcs.open(fn, "wb") as f:
+        f.write(b"zz")
+    assert gcs.info(fn)["contentType"] == "text/plain"
+
+
+def test_content_type_default(gcs):
+    fn = TEST_BUCKET + "/content_type.abcdef"
+    with gcs.open(fn, "wb") as f:
+        f.write(b"zz")
+    assert gcs.info(fn)["contentType"] == "application/octet-stream"
+
+
+def test_content_type_put_guess(gcs):
+    dst = TEST_BUCKET + "/content_type_put_guess"
+    with tmpfile(extension="txt") as fn:
+        with open(fn, "w") as f:
+            f.write("zz")
+        gcs.put(fn, f"gs://{dst}", b"")
+    assert gcs.info(dst)["contentType"] == "text/plain"
+
+
 def test_attrs(gcs):
     if not gcs.on_google:
         # https://github.com/fsspec/gcsfs/pull/479
@@ -913,8 +996,6 @@ def test_request_header(gcs):
 
 
 def test_user_project_fallback_google_default(monkeypatch):
-    import fsspec
-
     monkeypatch.setattr(gcsfs.core, "DEFAULT_PROJECT", "my_default_project")
     monkeypatch.setattr(fsspec.config, "conf", {})
     monkeypatch.setattr(
@@ -1003,10 +1084,16 @@ def test_put_small_cache_validity(gcs):
 def test_pseudo_dir_find(gcs):
     gcs.rm(f"{TEST_BUCKET}/*", recursive=True)
     gcs.touch(f"{TEST_BUCKET}/a/b/file")
+
+    c = gcs.glob(f"{TEST_BUCKET}/a/b/*")
+    assert c == [f"{TEST_BUCKET}/a/b/file"]
+
     b = set(gcs.glob(f"{TEST_BUCKET}/a/*"))
-    assert f"{TEST_BUCKET}/a/b" in b
+    assert b == {f"{TEST_BUCKET}/a/b"}
+
     a = set(gcs.glob(f"{TEST_BUCKET}/*"))
-    assert f"{TEST_BUCKET}/a" in a
+    assert a == {f"{TEST_BUCKET}/a"}
+
     assert gcs.find(TEST_BUCKET) == [f"{TEST_BUCKET}/a/b/file"]
     assert gcs.find(f"{TEST_BUCKET}/a", withdirs=True) == [
         f"{TEST_BUCKET}/a",
@@ -1137,7 +1224,6 @@ def test_dir_marker(gcs):
 
 
 def test_mkdir_with_path(gcs):
-
     with pytest.raises(FileNotFoundError):
         gcs.mkdir(f"{TEST_BUCKET + 'new'}/path", create_parents=False)
     assert not gcs.exists(f"{TEST_BUCKET + 'new'}")
@@ -1364,3 +1450,127 @@ def test_multiglob(gcs):
     assert gcs.ls(gparent, detail=False) == [f"{root}/t1/t2/t3"]
     gcs.glob(ggparent + "/")
     assert gcs.ls(gparent, detail=False) == [f"{root}/t1/t2/t3"]
+
+
+def test_expiry_keyword():
+    gcs = GCSFileSystem(listings_expiry_time=1, token="anon")
+    assert gcs.dircache.listings_expiry_time == 1
+    gcs = GCSFileSystem(cache_timeout=1, token="anon")
+    assert gcs.dircache.listings_expiry_time == 1
+    gcs = GCSFileSystem(cache_timeout=0, token="anon")
+    assert gcs.dircache.listings_expiry_time == 0
+
+
+def test_copy_cache_invalidated(gcs):
+    # Issue https://github.com/fsspec/gcsfs/issues/562
+    source = TEST_BUCKET + "/source"
+    gcs.mkdir(source)
+    gcs.touch(source + "/file2")
+
+    target = TEST_BUCKET + "/target"
+    assert not gcs.exists(target)
+    gcs.touch(target + "/dummy")
+    assert gcs.isdir(target)
+
+    target_file2 = target + "/file2"
+    gcs.cp(source + "/file2", target)
+
+    # Explicitly check that target has been removed from DirCache
+    assert target not in gcs.dircache
+
+    # Prior to fix the following failed as cache stale
+    assert gcs.isfile(target_file2)
+
+
+def test_transaction(gcs):
+    # https://github.com/fsspec/gcsfs/issues/389
+    if not gcs.on_google:
+        pytest.skip()
+    try:
+        with gcs.transaction:
+            with gcs.open(f"{TEST_BUCKET}/foo", "wb") as f:
+                f.write(b"This is a test string")
+            f.discard()
+            assert not gcs.exists(f"{TEST_BUCKET}/foo")
+            raise ZeroDivisionError
+    except ZeroDivisionError:
+        ...
+    assert not gcs.exists(f"{TEST_BUCKET}/foo")
+
+    with gcs.transaction:
+        with gcs.open(f"{TEST_BUCKET}/foo", "wb") as f:
+            f.write(b"This is a test string")
+        assert not gcs.exists(f"{TEST_BUCKET}/foo")
+
+    assert gcs.cat(f"{TEST_BUCKET}/foo") == b"This is a test string"
+
+
+def test_find_maxdepth(gcs):
+    assert gcs.find(f"{TEST_BUCKET}/nested", maxdepth=None) == [
+        f"{TEST_BUCKET}/nested/file1",
+        f"{TEST_BUCKET}/nested/file2",
+        f"{TEST_BUCKET}/nested/nested2/file1",
+        f"{TEST_BUCKET}/nested/nested2/file2",
+    ]
+
+    assert gcs.find(f"{TEST_BUCKET}/nested", maxdepth=None, withdirs=True) == [
+        f"{TEST_BUCKET}/nested",
+        f"{TEST_BUCKET}/nested/file1",
+        f"{TEST_BUCKET}/nested/file2",
+        f"{TEST_BUCKET}/nested/nested2",
+        f"{TEST_BUCKET}/nested/nested2/file1",
+        f"{TEST_BUCKET}/nested/nested2/file2",
+    ]
+
+    assert gcs.find(f"{TEST_BUCKET}/nested", maxdepth=1) == [
+        f"{TEST_BUCKET}/nested/file1",
+        f"{TEST_BUCKET}/nested/file2",
+    ]
+
+    assert gcs.find(f"{TEST_BUCKET}/nested", maxdepth=1, withdirs=True) == [
+        f"{TEST_BUCKET}/nested",
+        f"{TEST_BUCKET}/nested/file1",
+        f"{TEST_BUCKET}/nested/file2",
+        f"{TEST_BUCKET}/nested/nested2",
+    ]
+
+    with pytest.raises(ValueError, match="maxdepth must be at least 1"):
+        gcs.find(f"{TEST_BUCKET}/nested", maxdepth=0)
+
+
+def test_sign(gcs, monkeypatch):
+    file = TEST_BUCKET + "/test.jpg"
+    with gcs.open(file, "wb") as f:
+        f.write(b"This is a test string")
+    assert gcs.cat(file) == b"This is a test string"
+
+    # `sign` is creating a google Client on its own, it needs a realistically
+    # looking credentials file.
+    if not gcs.on_google:
+        monkeypatch.setenv(
+            "GOOGLE_APPLICATION_CREDENTIALS",
+            os.path.dirname(__file__) + "/fake-service-account-credentials.json",
+        )
+
+    current_ts_utc = int(datetime.now(tz=timezone.utc).timestamp())
+    result = gcs.sign(file)
+
+    # Check it here since emulator doesn't really validate those values
+    params = parse_qs(urlparse(result).query)
+    assert int(params["Expires"][0]) >= current_ts_utc + 100
+
+    response = requests.get(result)
+    assert response.text == "This is a test string"
+
+
+@pytest.mark.xfail(reason="emulator does not support condition")
+def test_write_x_mpu(gcs):
+    fn = TEST_BUCKET + "/test.file"
+    with gcs.open(fn, mode="xb", block_size=5 * 2**20) as f:
+        assert f.mode == "xb"
+        f.write(b"0" * 5 * 2**20)
+        f.write(b"done")
+    with pytest.raises(FileExistsError):
+        with gcs.open(fn, mode="xb", block_size=5 * 2**20) as f:
+            f.write(b"0" * 5 * 2**20)
+            f.write(b"done")
